let vm = new Vue({
    el: '#app',
    /***
     * In `data` mettiamo tutte quelle proprietà che servirano alla nostra app.
     * `books` sarà la lista dei libri da visualizzare nella schermata principale,
     * `book` sarà l'oggetto libro che useremo come rigerimento per creare/modificare nuovi libri e
     * `mode` lo usiamo per decidere cosa renderizzare.
     * Quest ultimo non è la soluzione migliore, sarebbe stato meglio usare delle componenti e dei template,
     * ma come esempio ce lo faremo andare bene.
     */
    data: {
        books: [],
        book: {},
        mode: 'main'
    },
    /**
     * `mounted` viene eseguito quando una componente è caricata e pronta. In questo caso
     * viene eseguita quando il DOM è pronto nella pagina principale.
     */
    mounted: function(){
        this.pageLoad();
    },
    /**
     * Qua mettiamo tutti i metodi che useremo nella nostra componente (in questo caso una sola: la app principale).
     * Notate come abbia messo asincrone le funzioni che usano `fetch()` in modo da non dover gestire le promise.
     * L'unica funzione "particolare" è `addBookMode`, la uso per caricare la form di inserimento dati senza caricare
     * un libro dal server. In pratica inizializza l'oggetto `book` ad un oggetto vuoto e cambia la modalità. Fine.
     */
    methods: {
        pageLoad: async function() {
            let data = await fetch("api/books/");
            this.books = await data.json();
            this.mode='main';
        },
        bookLoad: async function(id) {            
            let data = await fetch(`api/books/${id}`);
            this.mode = 'edit';
            this.book = await data.json();
        },
        backOnBooks: function(){
            this.mode = 'main';
        },
        addBookMode: function(){
            this.book = {};
            this.mode = 'add';
        },
        addBook: async function(){
            let data = await fetch("api/books/", {
                'method': 'POST',
                'headers': {
                    'Accept': 'application/json',
                    'Content-Type': 'application/json'
                },
                'body': JSON.stringify(this.book)
            });
            this.book = await data.json();
            this.pageLoad();            
        },
        editBook: async function(){
            let data = await fetch("api/books/", {
                'method': 'PUT',
                'headers': {
                    'Accept': 'application/json',
                    'Content-Type': 'application/json'
                },
                'body': JSON.stringify(this.book)
            });
            this.book = await data.json();
            this.pageLoad();            
        }
    }
})